@@ -39,9 +39,11 @@
 
 #include "powertop.h"
 
+#define VERSION "1.11"
+
 uint64_t start_usage[8], start_duration[8];
 uint64_t last_usage[8], last_duration[8];
-char cnames[8][10];
+char cnames[8][16];
 
 double ticktime = 15.0;
 
@@ -359,23 +361,26 @@ static void read_data_cpuidle(uint64_t * usage, uint64_t * duration)
 				fclose(file);
 				if (f == NULL)
 					break;
+
 			
 				f = strstr(line, "MWAIT ");
 				if (f) {
 					f += 6;
 					clevel = (strtoull(f, NULL, 16)>>4) + 1;
-				}
-				sprintf(cnames[clevel], "C%i", clevel);
+					sprintf(cnames[clevel], "C%i mwait", clevel);
+				} else
+					sprintf(cnames[clevel], "C%i\t", clevel);
+
 				f = strstr(line, "POLL IDLE");
 				if (f) {
 					clevel = 0;
-					sprintf(cnames[clevel], "%s", _("polling"));
+					sprintf(cnames[clevel], "%s\t", _("polling"));
 				}
 
 				f = strstr(line, "ACPI HLT");
 				if (f) {
 					clevel = 1;
-					sprintf(cnames[clevel], "%s", "C1 halt");
+					sprintf(cnames[clevel], "%s\t", "C1 halt");
 				}
 			}
 			sprintf(filename + len, "/%s/usage", entry->d_name);
@@ -477,7 +482,7 @@ void sort_lines(void)
 
 
 
-int print_battery_proc(void)
+int print_battery_proc_acpi(void)
 {
 	DIR *dir;
 	struct dirent *dirent;
@@ -564,6 +569,79 @@ int print_battery_proc(void)
 	return 1;
 }
 
+int print_battery_proc_pmu(void)
+{
+	char line[80];
+	int i;
+	int power_present = 0;
+	int num_batteries = 0;
+	/* unsigned rem_time_sec = 0; */
+	unsigned charge_mAh = 0, max_charge_mAh = 0, voltage_mV = 0;
+	int discharge_mA = 0;
+	FILE *fd;
+
+	fd = fopen("/proc/pmu/info", "r");
+	if (fd == NULL)
+		return 0;
+
+	while ( fgets(line, sizeof(line), fd) != NULL )
+	{
+		if (strncmp("AC Power", line, strlen("AC Power")) == 0)
+			sscanf(strchr(line, ':')+2, "%d", &power_present);
+		else if (strncmp("Battery count", line, strlen("Battery count")) == 0)
+			sscanf(strchr(line, ':')+2, "%d", &num_batteries);
+	}
+	fclose(fd);
+
+	for (i = 0; i < num_batteries; ++i)
+	{
+		char file_name[20];
+		int flags = 0;
+		/* int battery_charging, battery_full; */
+		/* unsigned this_rem_time_sec = 0; */
+		unsigned this_charge_mAh = 0, this_max_charge_mAh = 0;
+		unsigned this_voltage_mV = 0, this_discharge_mA = 0;
+
+		snprintf(file_name, sizeof(file_name), "/proc/pmu/battery_%d", i);
+		fd = fopen(file_name, "r");
+		if (fd == NULL)
+			continue;
+
+		while (fgets(line, sizeof(line), fd) != NULL)
+		{
+			if (strncmp("flags", line, strlen("flags")) == 0)
+				sscanf(strchr(line, ':')+2, "%x", &flags);
+			else if (strncmp("charge", line, strlen("charge")) == 0)
+				sscanf(strchr(line, ':')+2, "%d", &this_charge_mAh);
+			else if (strncmp("max_charge", line, strlen("max_charge")) == 0)
+				sscanf(strchr(line, ':')+2, "%d", &this_max_charge_mAh);
+			else if (strncmp("voltage", line, strlen("voltage")) == 0)
+				sscanf(strchr(line, ':')+2, "%d", &this_voltage_mV);
+			else if (strncmp("current", line, strlen("current")) == 0)
+				sscanf(strchr(line, ':')+2, "%d", &this_discharge_mA);
+			/* else if (strncmp("time rem.", line, strlen("time rem.")) == 0) */
+			/*   sscanf(strchr(line, ':')+2, "%d", &this_rem_time_sec); */
+		}
+		fclose(fd);
+
+		if ( !(flags & 0x1) )
+			/* battery isn't present */
+			continue;
+
+		/* battery_charging = flags & 0x2; */
+		/* battery_full = !battery_charging && power_present; */
+
+		charge_mAh += this_charge_mAh;
+		max_charge_mAh += this_max_charge_mAh;
+		voltage_mV += this_voltage_mV;
+		discharge_mA += this_discharge_mA;
+		/* rem_time_sec += this_rem_time_sec; */
+	}
+	show_pmu_power_line(voltage_mV, charge_mAh, max_charge_mAh,
+	                    discharge_mA);
+	return 1;
+}
+
 void print_battery_sysfs(void)
 {
 	DIR *dir;
@@ -574,7 +652,10 @@ void print_battery_sysfs(void)
 
 	char filename[256];
 	
-	if (print_battery_proc())
+	if (print_battery_proc_acpi())
+		return;
+	
+	if (print_battery_proc_pmu())
 		return;
 
 	dir = opendir("/sys/class/power_supply");
@@ -627,12 +708,19 @@ void print_battery_sysfs(void)
 
 		sprintf(filename, "/sys/class/power_supply/%s/energy_now", dirent->d_name);
 		file = fopen(filename, "r");
-		if (!file)
-			continue;
-		memset(line, 0, 1024);
-		if (fgets(line, 1024, file) != NULL) {
-			watts_left = strtoull(line, NULL, 10) / 1000000.0;
+		watts_left = 1;
+		if (!file) {
+			sprintf(filename, "/sys/class/power_supply/%s/charge_now", dirent->d_name);
+			file = fopen(filename, "r");
+			if (!file) 
+				continue;
+
+			/* W = A * V */
+			watts_left = voltage;
 		}
+		memset(line, 0, 1024);
+		if (fgets(line, 1024, file) != NULL) 
+			watts_left *= strtoull(line, NULL, 10) / 1000000.0;
 		fclose(file);
 
 		sprintf(filename, "/sys/class/power_supply/%s/current_now", dirent->d_name);
@@ -641,7 +729,7 @@ void print_battery_sysfs(void)
 			continue;
 		memset(line, 0, 1024);
 		if (fgets(line, 1024, file) != NULL) {
-			watts_drawn = strtoull(line, NULL, 10) / 1000000.0;
+			amperes_drawn = strtoull(line, NULL, 10) / 1000000.0;
 		}
 		fclose(file);
 	
@@ -677,6 +765,13 @@ void usage()
 	printf(_("  -d, --dump            read wakeups once and print list of top offenders\n"));
 	printf(_("  -t, --time=DOUBLE     default time to gather data in seconds\n"));
 	printf(_("  -h, --help            Show this help message\n"));
+	printf(_("  -v, --version         Show version information and exit\n"));
+	exit(0);
+}
+
+void version()
+{
+	printf(_("powertop version %s\n"), VERSION);
 	exit(0);
 }
 
@@ -697,11 +792,12 @@ int main(int argc, char **argv)
  			{ "dump", 0, NULL, 'd' },
  			{ "time", 1, NULL, 't' },
  			{ "help", 0, NULL, 'h' },
+ 			{ "version", 0, NULL, 'v' },
  			{ 0, 0, NULL, 0 }
  		};
  		int index2 = 0, c;
  		
- 		c = getopt_long(argc, argv, "dt:h", opts, &index2);
+ 		c = getopt_long(argc, argv, "dt:hv", opts, &index2);
  		if (c == -1)
  			break;
  		switch (c) {
@@ -714,6 +810,9 @@ int main(int argc, char **argv)
  		case 'h':
  			usage();
  			break;
+ 		case 'v':
+ 			version();
+ 			break;
  		default:
  			;
  		}
@@ -737,7 +836,7 @@ int main(int argc, char **argv)
 
 	memset(cur_usage, 0, sizeof(cur_usage));
 	memset(cur_duration, 0, sizeof(cur_duration));
-	printf("PowerTOP 1.10    (C) 2007, 2008 Intel Corporation \n\n");
+	printf("PowerTOP " VERSION "   (C) 2007, 2008 Intel Corporation \n\n");
 	if (geteuid() != 0)
 		printf(_("PowerTOP needs to be run as root to collect enough information\n"));
 	printf(_("Collecting data for %i seconds \n"), (int)ticktime);
@@ -819,7 +918,7 @@ int main(int argc, char **argv)
 					if (cnames[i][0]==0)
 						sprintf(cnames[i],"C%i",i+1);
 					sprintf
-					    (cstate_lines[2+i], _("%s\t\t%5.1fms (%4.1f%%)\n"),
+					    (cstate_lines[2+i], _("%s\t%5.1fms (%4.1f%%)\n"),
 					     cnames[i], sleept, percentage);
 					if (maxsleep < sleept)
 						maxsleep = sleept;
@@ -938,8 +1037,11 @@ int main(int argc, char **argv)
 
 		if (key) {
 			char keychar;
+			int keystroke = fgetc(stdin);
+			if (keystroke == EOF)
+				exit(EXIT_SUCCESS);
 
-			keychar = toupper(fgetc(stdin));
+			keychar = toupper(keystroke);
 			if (keychar == 'Q')
 				exit(EXIT_SUCCESS);
 			if (keychar == 'R')
@@ -1015,7 +1117,7 @@ int main(int argc, char **argv)
 		/* suggest to stop hal polilng if it shows up in the top 50 and wakes up too much*/
 		suggest_process_death("hald-addon-stor : ", "hald-addon-storage", lines, min(linehead,50), 2.0,
 				    _( "Suggestion: Disable 'hal' from polling your cdrom with:  \n" 
-				       "hal-disable-polling --device /dev/scd0 'hal' is the component that auto-opens a\n"
+				       "hal-disable-polling --device /dev/cdrom 'hal' is the component that auto-opens a\n"
 				       "window if you plug in a CD but disables SATA power saving from kicking in."), 30);
 
 		/* suggest to kill sealert; it wakes up 10 times/second on a default F7 install*/
