@@ -54,7 +54,26 @@ static char wireless_nic[32];
 static char rfkill_path[PATH_MAX];
 static char powersave_path[PATH_MAX];
 
-static int check_unused_wiresless_up(void)
+static int rfkill_enabled(void)
+{
+	FILE *file;
+	char val;
+	if (strlen(rfkill_path)<2)
+		return 0;
+	if (access(rfkill_path, W_OK))
+		return 0;
+
+	file = fopen(rfkill_path, "r");
+	if (!file)
+		return 0;
+	val = fgetc(file);
+	fclose(file);
+	if (val != '0') /* already rfkill'd */
+		return 1;
+	return 0;
+}
+
+int check_unused_wiresless_up(void)
 {
 	FILE *file;
 	char val;
@@ -96,6 +115,9 @@ static int need_wireless_suggest(char *iface)
 	char line[1024];
 	int ret = 0;
 
+	if (rfkill_enabled())
+		return 0;
+
 	sprintf(line, "/sbin/iwpriv %s get_power 2> /dev/null", iface);
 	file = popen(line, "r");
 	if (!file)
@@ -123,6 +145,9 @@ static int need_wireless_suggest_new(void)
 	if (access(powersave_path, W_OK))
 		return 0;
 
+	if (rfkill_enabled())
+		return 0;
+
 	file = fopen(powersave_path, "r");
 	if (!file)
 		return 0;
@@ -134,9 +159,45 @@ static int need_wireless_suggest_new(void)
 	return 1;
 }
 
+void find_4965(void)
+{
+	static int tried_4965 = 0;
+	DIR *dir;
+	struct dirent *dirent;
+	char pathname[PATH_MAX];
+
+	if (tried_4965++)
+		return;
+
+	dir = opendir("/sys/bus/pci/drivers/iwl4965");
+	while (dir && (dirent = readdir(dir))) {
+		if (dirent->d_name[0]=='.')
+			continue;
+		sprintf(pathname, "/sys/bus/pci/drivers/iwl4965/%s/power_level", dirent->d_name);
+		if (!access(pathname, W_OK))
+			strcpy(powersave_path, pathname);
+	}
+	if (dir)
+		closedir(dir);
+	dir = opendir("/sys/bus/pci/drivers/iwl3945");
+	if (!dir)
+		return;
+	while ((dirent = readdir(dir))) {
+		if (dirent->d_name[0]=='.')
+			continue;
+		sprintf(pathname, "/sys/bus/pci/drivers/iwl3945/%s/power_level", dirent->d_name);
+		if (!access(pathname, W_OK))
+			strcpy(powersave_path, pathname);
+	}
+
+	closedir(dir);
+
+}
+
 
 void find_wireless_nic(void) 
 {
+	static int found = 0;
 	FILE *file;
 	int sock;
 	struct ifreq ifr;
@@ -145,10 +206,15 @@ void find_wireless_nic(void)
 	int ifaceup = 0;
 	int ret;
 
+	if (found++)
+		return;
+
 	wireless_nic[0] = 0;
 	rfkill_path[0] = 0;
 	powersave_path[0] = 0;
 
+	strcpy(wireless_nic, "wlan0");
+
 	file = popen("/sbin/iwpriv -a 2> /dev/null", "r");
 	if (!file)
 		return;
@@ -163,6 +229,8 @@ void find_wireless_nic(void)
 			if (c) *c = 0;
 			strcpy(wireless_nic, line);
 		}
+		if (strstr(line, "wlan0:"))
+			strcpy(wireless_nic, "wlan0");
 	}
 	pclose(file);
 
@@ -196,7 +264,7 @@ void find_wireless_nic(void)
         ifr.ifr_data = (void*) &driver;
         ret = ioctl(sock, SIOCETHTOOL, &ifr);
 
-	sprintf(rfkill_path,"/sys/bus/pci/devices/%s/rf_kill", driver.bus_info);
+	sprintf(rfkill_path,"/sys/bus/pci/devices/%s/rfkill/rfkill0/state", driver.bus_info);
 	sprintf(powersave_path,"/sys/bus/pci/devices/%s/power_level", driver.bus_info);
 	close(sock);
 }
@@ -213,7 +281,7 @@ void activate_wireless_suggestion_new(void)
 	file = fopen(powersave_path, "w");
 	if (!file)
 		return;
-	fprintf(file,"5\n");
+	fprintf(file,"1\n");
 	fclose(file);
 }
 
@@ -231,8 +299,9 @@ void suggest_wireless_powersave(void)
 	char sug[1024];
 	int ret;
 
-	if (strlen(wireless_nic)==0)
+	if (strlen(wireless_nic)==0) 
 		find_wireless_nic();
+	find_4965();
 	ret = check_unused_wiresless_up();
 
 	if (ret >= 0 && need_wireless_suggest(wireless_nic)) {
